def boop(repo, node, predecessors):
    # XXX we might need a tree of markers :(
    #       3
    #        \
    # 0 - 1 - 2 - 5 - 6
    #        /
    #       4
    # 6 was rewritten from 0 using amend (and fold)???

    """

    >>> list(boop({0, 6}, 6, {
    ...     1: [(0, [1], {})],
    ...     2: [(1, [2], {'fold-id': 'foo'}), (3, [2], {'fold-id': 'foo'}), (4, [2], {'fold-id': 'foo'})],
    ...     5: [(2, [5], {})],
    ...     6: [(5, [6], {})],
    ... }))
    [((0,), {})]
    >>> boop()
    False
    """
    stack = [node]
    seen = set(stack)
    markers = set()

    while stack:
        current = stack.pop()
        currentpreds = predecessors.get(current, ())

        # Get all fold-related markers for current node, they either get added
        # to the rest of markers XXX or they all get discarded XXX ???
        foldmarkers = {
            pred for pred in currentpreds
            if any(v for (k, v) in pred[3] if k == b'fold-id')
        }

        for pred in currentpreds:
            prednode = pred[0]

            if pred not in foldmarkers:
                markers.add(pred)

            # Basic cycle protection
            if prednode in seen:
                continue
            seen.add(prednode)

            if prednode in repo:
                yield (prednode,), markers | foldmarkers
                markers = set()
            else:
                stack.append(prednode)



# fqbn.py - code related to FQBN (fully qualified branch name)
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
from mercurial import (
    util,
)

class fqbn(object):
    def __init__(self, s):
        self._value = s

    @util.propertycache
    def branch(self):
        return b'default'

    @util.propertycache
    def topic_namespace(self):
        return b'none'

    @util.propertycache
    def topic(self):
        return b''

    @util.propertycache
    def short(self):
        return b''

    @util.propertycache
    def normal(self):
        return b''

    @util.propertycache
    def full(self):
        return b''
